macro(emiplib_support_option DESCRIPTION OPTIONNAME DEFINENAME DEFAULTVALUE EMPTYVALUE)
	option(${OPTIONNAME} ${DESCRIPTION} ${DEFAULTVALUE})
	if (${OPTIONNAME})
		set(${DEFINENAME} "#define ${DEFINENAME}")
	else (${OPTIONNAME})
		set(${DEFINENAME} "${EMPTYVALUE}")
	endif (${OPTIONNAME})
endmacro(emiplib_support_option)

macro (emiplib_test_feature_option FILENAME DEFINENAME EMPTYVALUE OPTIONNAME OPTIONMESSAGE) 
	if (NOT DEFINED ${FILENAME}_RESULT)
		message(STATUS "Compiling ${FILENAME}.cpp")
		try_compile(${FILENAME}_RESULT "${PROJECT_BINARY_DIR}" "${PROJECT_SOURCE_DIR}/tools/${FILENAME}.cpp"
			    OUTPUT_VARIABLE OUTVAR)
	endif (NOT DEFINED ${FILENAME}_RESULT)
	if (${FILENAME}_RESULT)
		set(BLA ON)
	else (${FILENAME}_RESULT)
		set(BLA OFF)
	endif (${FILENAME}_RESULT)

	option(${OPTIONNAME} ${OPTIONMESSAGE} ${BLA})
	if (${OPTIONNAME})
		set(${DEFINENAME} "#define ${DEFINENAME}")
	else (${OPTIONNAME})
		set(${DEFINENAME} "${EMPTYVALUE}")
	endif (${OPTIONNAME})
endmacro (emiplib_test_feature_option)

macro(save_paths VARNAME)
	set (BLA "${ARGN}")
	foreach(i IN LISTS BLA)
		set (BLA2 "${i}")
		if (BLA2)
			list(APPEND ${VARNAME} "${i}")
		endif (BLA2)
	endforeach(i)
	list(LENGTH ${VARNAME} BLA)
	if (BLA GREATER 0)
		list(REMOVE_DUPLICATES ${VARNAME})
	endif (BLA GREATER 0)
endmacro(save_paths)

macro(remove_empty VARNAME)
	set (remove_empty_NEWLIST "")
	foreach(i IN LISTS ${VARNAME})
		set (BLA2 "${i}")
		if (BLA2)
			list(APPEND remove_empty_NEWLIST "${i}")
		endif (BLA2)
	endforeach(i)
	set(${VARNAME} "${remove_empty_NEWLIST}")
endmacro(remove_empty)

macro(apply_include_paths VARNAME)
	set (BLA "${VARNAME}")
	foreach(i IN LISTS BLA)
		set (BLA2 "${i}")
		if (BLA2)
			include_directories("${i}")
		endif (BLA2)
	endforeach(i)
endmacro(apply_include_paths)

macro(add_additional_stuff INCVAR LIBVAR)
	set(ADDITIONAL_INCLUDE_DIRS "" CACHE STRING "Additional include directories")
	if (UNIX AND NOT WIN32)
		set(ADDITIONAL_LIBRARIES "" CACHE STRING "Additional libraries to link against")
	else (UNIX AND NOT WIN32)
		set(ADDITIONAL_LIBRARIES_GENERAL "" CACHE STRING "Additional libraries to link against in both debug and release modes")
		set(ADDITIONAL_LIBRARIES_RELEASE "" CACHE STRING "Additional libraries to link against in release mode")
		set(ADDITIONAL_LIBRARIES_DEBUG "" CACHE STRING "Additional libraries to link against in debug mode")

		set(ADDITIONAL_LIBRARIES "${ADDITIONAL_LIBRARIES_GENERAL}")

		foreach(l IN LISTS ADDITIONAL_LIBRARIES_RELEASE)
			list(APPEND ADDITIONAL_LIBRARIES optimized)
			list(APPEND ADDITIONAL_LIBRARIES "${l}")
		endforeach(l)
		foreach(l IN LISTS ADDITIONAL_LIBRARIES_DEBUG)
			list(APPEND ADDITIONAL_LIBRARIES debug)
			list(APPEND ADDITIONAL_LIBRARIES "${l}")
		endforeach(l)
	endif (UNIX AND NOT WIN32)

	save_paths(${INCVAR} "${ADDITIONAL_INCLUDE_DIRS}")
	save_paths(${LIBVAR} "${ADDITIONAL_LIBRARIES}")
endmacro(add_additional_stuff)

